<?php

/* ************************************************* *\
|* ================================================= *|
|* =       Copyright (c) 2012 Jacob Pollack.       = *|
|* ================================================= *|
|* = THIS SOFTWARE IS PRIVATE AND CANNOT BE FOUND  = *|
|* = ON THE PUBLIC DOMAIN. ANY DISTRIBUTION OR USE = *|
|* = OF THIS SOFTWARE WITHOUT THE EXPLICIT         = *|
|* = PERMISSION OF THE AUTHOR WILL RESULT IN LEGAL = *|
|* = ACTION BEING TAKEN.                           = *|
|* =                                               = *|
|* = THIS SOFTWARE IS CREATED FOR EDUCATIONAL      = *|
|* = PURPOSES ONLY. CHANGES OF ANY SORT MUST BE    = *|
|* = SUBMITTED TO THE RESPOSITORY ACCORDINGLY.     = *|
|* ================================================= *|
|* = @author    Jacob Pollack                      = *|
|* = @email     jacobpollack@jamach.com            = *|
|* = @date      05/08/2012 (DD/MM/YYYY)            = *|
|* =                                               = *|
|* = @contributors  none                           = *|
|* ================================================= *|
\*************************************************** */

/**
 * This is the initialization file. Instantiates all of
 * the objects and invokes the core functionality of the
 * software.
 * 
 * @name    Initialization
 * @author  Jacob Pollack
 */

/**
 * Defines important folder location constants.
 */
define('CWD', ($cwd = getcwd() . '\\') ? $cwd : '.\\');
define('CACHE', 'cache/');
define('METHODS', 'methods/');
define('MODULES', 'modules/');
define('TPL', 'tpl/');
define('UTIL', 'util/');
define('WRAPPERS', 'wrappers/');
define('NODES', 'nodes/');

/**
 * Defines important module constants.
 */
define('USE_MODULES', true);

/**
 * Includes the core utilities.
 */
require_once CWD . INCLUDES . UTIL . 'error.php';
require_once CWD . INCLUDES . UTIL . 'arrays.php';
require_once CWD . INCLUDES . UTIL . 'gc.php';
require_once CWD . INCLUDES . UTIL . 'casters.php';
require_once CWD . INCLUDES . UTIL . 'loader.php';

/**
 * Include the core methods.
 */
require_once CWD . INCLUDES . METHODS . 'storage.php';

/**
 * Checks the server environment.
 */
if (version_compare(PHP_VERSION, '5.3.5') < 0)
{
    trigger_error('There is an issue with your PHP version. Ensure you are running at least PHP 5.3.5.');
}

if ((defined('CWD') == false) || (CWD == '.\\'))
{
    define('CWD', ($cwd = getcwd() . '\\') ? $cwd : '.\\');
}

/**
 * Instantiates the appropriate core objects.
 */
if (class_exists('environment') == true)
{
    $object = new stdClass();
    
    if (is_object($object) == true)
    {
        $quasar = new environment($object);
        
        if (class_exists('cachehandler') == true)
        {
            $quasar->object->cachehandler = new cachehandler(CACHE);
        }
        
        $quasar->load_configuration();
        
        if (class_exists('storage') == true)
        {
            $quasar->object->storage = new storage();
            
            $quasar->object->storage->initialize($quasar->configuration);
            
            // Deletes the authentication object (important).
            delete($quasar->configuration->storage->authentication);
        }
        
        if (class_exists('templatehandler') == true)
        {
            $quasar->object->templatehandler = new templatehandler(TPL);
        }
    }
}

/**
 * Instantiates the Module Handler object and loads all the modules.
 */
if (class_exists('modulehandler') == true)
{
    $quasar->object->modulehandler = new modulehandler(USE_MODULES);
}

?>