<?php

/* ************************************************* *\
|* ================================================= *|
|* =       Copyright (c) 2012 Jacob Pollack.       = *|
|* ================================================= *|
|* = THIS SOFTWARE IS PRIVATE AND CANNOT BE FOUND  = *|
|* = ON THE PUBLIC DOMAIN. ANY DISTRIBUTION OR USE = *|
|* = OF THIS SOFTWARE WITHOUT THE EXPLICIT         = *|
|* = PERMISSION OF THE AUTHOR WILL RESULT IN LEGAL = *|
|* = ACTION BEING TAKEN.                           = *|
|* =                                               = *|
|* = THIS SOFTWARE IS CREATED FOR EDUCATIONAL      = *|
|* = PURPOSES ONLY. CHANGES OF ANY SORT MUST BE    = *|
|* = SUBMITTED TO THE RESPOSITORY ACCORDINGLY.     = *|
|* ================================================= *|
|* = @author    Jacob Pollack                      = *|
|* = @email     jacobpollack@jamach.com            = *|
|* = @date      05/08/2012 (DD/MM/YYYY)            = *|
|* =                                               = *|
|* = @contributors  none                           = *|
|* ================================================= *|
\*************************************************** */

/**
 * This is the storage file. Contains all the methods and 
 * PHP native functions for interacting with the appropriate
 * storage driver.
 * 
 * @name    Storage
 * @author  Jacob Pollack
 * 
 * @deprecated  The MySQL storage class will be deprecated in future versions of PHP.
 */

final class storage
{
    /**
     * Connection status to the storage driver.
     * 
     * @var boolean
     */
    private $connected = false;
    
    /**
     * Link for the MySQL driver.
     * 
     * @var link
     */
    private $link = null;
    
    /**
     * A key -> value array containing all of the PHP
     * native functions for the MySQL driver.
     * 
     * @var array
     */
    private $function = array(
    'connect'               =>              'mysql_connect',
    'pconnect'              =>              'mysql_pconnect',
    'select_db'             =>              'mysql_select_db',
    'query'                 =>              'mysql_query',
    'unbuffered_query'      =>              'mysql_unbuffered_query',
    'num_rows'              =>              'mysql_num_rows',
    'fetch_object'          =>              'mysql_fetch_object'
    );
    
    /**
     * Configuration object.
     * 
     * @var object  (stdClass)
     */
    private $configuration = null;
    
    /**
     * Empty constructor.
     * 
     * @return  void
     */
    final public function __construct() { }
    
    /**
     * Initializes the storage driver(s) from the configuration and begins
     * the database activity.
     * 
     * @param   stdClass    $configuration      The configuration object passed by reference containing all of the configuration information.
     * 
     * @return  void
     */
    final public function initialize(stdClass &$configuration)
    {
        $this->configuration = &$configuration;
        
        //if (is_object($this->configuration) == true)
        //{
            $this->connect();
            
            if ($this->connected == true)
            {
                $this->select_database($this->link);
            }
        //}
        //else
        //{
        //    trigger_error('Problem with the configuration object.');
        //}
    }
    
    /**
     * Establishes a connection with the MySQL storage driver.
     * 
     * @return  void
     */
    final private function connect()
    {
        if ($this->connected == false)
        {
            $this->link = $this->function[(($this->configuration->storage->authentication->persistant == true) ? 'pconnect': 'connect')]($this->configuration->storage->authentication->hostname . ':' . $this->configuration->storage->authentication->port,
                                                                                                                                            $this->configuration->storage->authentication->username,
                                                                                                                                            $this->configuration->storage->authentication->password);
            
            if ($this->link)
            {
                $this->connected = true;
            }
        }
    }
    
    /**
     * Selects a database to access in the MySQL driver.
     * 
     * @param:  link    $link   MySQL link passed by reference.
     * 
     * @return  void
     */
    final private function select_database(&$link)
    {
        $this->function['select_db']($this->configuration->storage->authentication->database, $link);
    }
    
    /**
     * Executes a database query (multiple queries not supported). The query
     * can be buffered and unbuffered.
     * 
     * @param   string      $sql        The SQL query to be executed.
     * @param   boolean     $buffered   Whether the query is buffered or not.
     * @param   resource    $link       The MySQL resource link.
     * 
     * @return  resource
     * @return  boolean
     * @return  data resource
     */
    final public function execute_query($sql, $buffered = true, $link = null)
    {
        if ($link == null)
        {
            $link = &$this->link;
        }
        
        return $this->function[(($buffered == true) ? 'query' : 'unbuffered_query')]($sql, $link);
    }
    
    /**
     * Returns the number of rows returned by a query.
     * 
     * @param   resource    $resource   The SQL query resource.
     * 
     * @return  integer
     */
    final public function num_rows($resource)
    {
        return $this->function['num_rows']($resource);
    }
    
    /**
     * Returns a result set as an object.
     * 
     * @param   resource    $resource   The SQL query resource passed by reference.
     * 
     * @return  object
     */
    final public function fetch_object(&$resource)
    {
        return $this->function['fetch_object']($resource);
    }
    
    /**
     * Empty destructor.
     * 
     * @return  void
     */
    final public function __destruct() { }
}

?>